
/*
 * RTC 디바이스 드라이버
 * 
 * 만든사람: sukistyle freestyle, suh1978@gmail.com
 * 개정내용:
 * 참고사항: rtc를 tick으로 사용하지는 않는다.
 *            soc 메뉴얼에서 보면 파워오프시에 RTCCEN을 0으로 두어야
 *            RTC레지스터에 잘못된 데이터가 write되는것을 방지한다고 한다.
 *            rtc관련 처리를 할때는 rtc_init/rtc_close를 꼭 함께 호출해야한다
 */

#include "rtc.h"
#include "interrupt.h"
#include "printk.h"
#include "string.h"

/*
 * Period = (n+1) / 128 second
 *
 * tick timer count를 127로 주었으니까 결국 매초마다 한번씩 인터럽트를 발생한다.
 * 처음 계획은 RTC를 메인 타이머로 사용할려고 했으나, 굳이 그럴필요 없을거 같아서
 * 그냥 현재 시간을 보여주는 용도로만 사용한다. 하지만 언제 또 마음이 변할지는 모르지...
 */
void rtc_init()
{
    rINTMSK &= ~(1<<8);
    rSRCPND |=   1<<8;
    rINTPND |=   1<<8;
    rTICNT   =  (TICK_INT_ENABLE | TICK_TIME_COUNT);
    rRTCCON |=   RTCEN;
}

/*
 * Power Off시에 임의로 RTC레지스터들에 쓰레기값이 들어가지 못하도록
 * RTC 를 차단시킨다
 *
 * 나는 RTC를 SWI를 통해서 현재시각을 알아오는 용도로 사용할것이므로
 * 해당 SWI_handler에서 rtc_init, rtc_get, rtc_close를 차례로 호출할것이다.
 */
void rtc_close()
{
    rRTCCON &= (unsigned char) ~(0x1);	/* RTC control disable				 */
}

void rtc_get()
{
    unsigned int sec, min, hour, date, day, mon, year;
    unsigned char DAY[4];

    sec  = rBCDSEC  & 0x7f;
    min  = rBCDMIN  & 0x7f;
    hour = rBCDHOUR & 0x3f;
    date = rBCDDATE & 0x3f; 
    day  = rBCDDAY  & 0x7;
    mon  = rBCDMON  & 0x1f;
    year = rBCDYEAR & 0xff;

    sec  = ((sec  >> 4) * 10) + (sec  & 0xf);
    min  = ((min  >> 4) * 10) + (min  & 0xf);
    hour = ((hour >> 4) * 10) + (hour & 0xf);
    date = ((date >> 4) * 10) + (date & 0xf);
    mon  = ((mon  >> 4) * 10) + (mon  & 0xf);
    year += 2000;

    switch (day) {

    case 1:
	strcpy(DAY, "MON");
	break;
	
    case 2:
	strcpy(DAY, "TUE");
	break;
	
    case 3:
	strcpy(DAY, "WED");
	break;
	
    case 4:
	strcpy(DAY, "THU");
	break;

    case 5:
	strcpy(DAY, "FRI");
	break;
	
    case 6:
	strcpy(DAY, "SAT");
	break;

    case 7:
	strcpy(DAY, "SUN");
	break;
	
    default:
	strcpy(DAY, "ERR");
	break;
    }
	
    printk("%d. %d. %d. (%s) %d:%d:%d\n", year, mon, date, DAY, hour, min, sec);
}

void rtc_set()
{
    unsigned char sec, min, hour, date, day, mon, year;
    unsigned char s[10];
    
    printk("SEC input number 0 ~ 59 : ");
    uart_gets(s, 10);
    sec = atoi(s);
    
    printk("\nMIN input number 0 ~ 59 : ");
    uart_gets(s, 10);
    min = atoi(s);

    printk("\nHOUR input number 0 ~ 23 : ");
    uart_gets(s, 10);
    hour = atoi(s);

    printk("\nDATE input number 1 ~ 31 : ");
    uart_gets(s, 10);
    date = atoi(s);
    
    printk("\nDAY input number 1 ~ 7 (1: MON, 2:TUE, 6:SAT, 7:SUN) : ");
    uart_gets(s, 10);
    day = atoi(s);

    printk("\nMON input number 1 ~ 12 : ");
    uart_gets(s, 10);
    mon = atoi(s);
    
    printk("\nYEAR input number 00 ~ 99 (00:2000, 1:2001) : ");
    uart_gets(s, 10);
    year = atoi(s);

    printk("%d. %d. %d. (%d) %d:%d:%d\n", year, mon, date, day, hour, min, sec);

    rBCDSEC  = ((sec  / 10) << 4) + (sec  % 10);
    rBCDMIN  = ((min  / 10) << 4) + (min  % 10);
    rBCDHOUR = ((hour / 10) << 4) + (hour % 10);
    rBCDDATE = ((date / 10) << 4) + (date % 10);
    rBCDMON  = ((mon  / 10) << 4) + (mon  % 10);
    rBCDYEAR = year & 0xff;

    rBCDDAY  = day;    
}
